#include "X3D_NurbsSurface.h"
#include "X3D_X3DTextureCoordinateNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NurbsSurface::NurbsSurface()
  : X3DParametricGeometryNode(),
    _controlPoint(MFVec3f()),
    _texCoord    (0),
    _uTesselation(0),
    _vTesselation(0),
    _weight      (MFDouble()),
    _ccw         (true),
    _solid       (true),
    _uDimension  (0),
    _vDimension  (0),
    _uKnot       (MFDouble()),
    _vKnot       (MFDouble()),
    _uOrder      (3),
    _vOrder      (3)
{
  defineTypeName("NurbsSurface", "NURBS");
}

NurbsSurface::NurbsSurface(const MFVec3f                  &controlPoint,
                           X3DTextureCoordinateNode       *texCoord,
                           const SFInt32                        &uTesselation,
                           const SFInt32                        &vTesselation,
                           const MFDouble                 &weight,
                           const SFBool                         &ccw,
                           const SFBool                         &solid,
                           const SFInt32                        &uDimension,
                           const SFInt32                        &vDimension,
                           const MFDouble                 &uKnot,
                           const MFDouble                 &vKnot,
                           const SFInt32                        &uOrder,
                           const SFInt32                        &vOrder)
  : X3DParametricGeometryNode(),
    _controlPoint(controlPoint),
    _texCoord    (0),
    _uTesselation(uTesselation),
    _vTesselation(vTesselation),
    _weight      (weight),
    _ccw         (ccw),
    _solid       (solid),
    _uDimension  (uDimension),
    _vDimension  (vDimension),
    _uKnot       (uKnot),
    _vKnot       (vKnot),
    _uOrder      (uOrder),
    _vOrder      (vOrder)
{
  defineTypeName("NurbsSurface", "NURBS");
  
  setTexCoord(static_cast<X3DTextureCoordinateNode *>(texCoord));
}

NurbsSurface::NurbsSurface(const NurbsSurface &N)
  : X3DParametricGeometryNode(N),
    _controlPoint(N._controlPoint),
    _texCoord    (0),
    _uTesselation(N._uTesselation),
    _vTesselation(N._vTesselation),
    _weight      (N._weight),
    _ccw         (N._ccw),
    _solid       (N._solid),
    _uDimension  (N._uDimension),
    _vDimension  (N._vDimension),
    _uKnot       (N._uKnot),
    _vKnot       (N._vKnot),
    _uOrder      (N._uOrder),
    _vOrder      (N._vOrder)
{
  setTexCoord(static_cast<X3DTextureCoordinateNode *>(N._texCoord));
}

SFAbstractNode NurbsSurface::clone() const
{
  return new NurbsSurface(*this);
}

NurbsSurface::~NurbsSurface()
{
  removeParentFromChild(this, _texCoord);
}

void NurbsSurface::setControlPoint(const MFVec3f &controlPoint)
{
  _controlPoint = controlPoint;
}

void NurbsSurface::setTexCoord(X3DTextureCoordinateNode *texCoord)
{
  removeParentFromChild(this, _texCoord);
  _texCoord = texCoord;
  addParentToChild(this, _texCoord);
}

void NurbsSurface::setUTesselation(const SFInt32 &uTesselation)
{
  _uTesselation = uTesselation;
}

void NurbsSurface::setVTesselation(const SFInt32 &vTesselation)
{
  _vTesselation = vTesselation;
}

void NurbsSurface::setWeight(const MFDouble &weight)
{
  _weight = weight;
}

void NurbsSurface::setCcw(const SFBool &ccw)
{
  _ccw = ccw;
}

void NurbsSurface::setSolid(const SFBool &solid)
{
  _solid = solid;
}

void NurbsSurface::setUDimension(const SFInt32 &uDimension)
{
  _uDimension = uDimension;
}

void NurbsSurface::setVDimension(const SFInt32 &vDimension)
{
  _vDimension = vDimension;
}

void NurbsSurface::setUKnot(const MFDouble &uKnot)
{
  _uKnot = uKnot;
}

void NurbsSurface::setVKnot(const MFDouble &vKnot)
{
  _vKnot = vKnot;
}

void NurbsSurface::setUOrder(const SFInt32 &uOrder)
{
  _uOrder = uOrder;
}

void NurbsSurface::setVOrder(const SFInt32 &vOrder)
{
  _vOrder = vOrder;
}

bool NurbsSurface::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DTextureCoordinateNode *>(N) != 0)
  {
    if (_texCoord != 0)
      return false;
    
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));   
    return true;    
  }
  return false;
}

bool NurbsSurface::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DTextureCoordinateNode *>(N) != 0)
  {
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));
    return true;    
  }
  if (N != 0)
    cx3d << "X3D::NurbsSurface::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::NurbsSurface::addChild : cannot add a NULL node!" << endl;
      
  
  return false;
}

bool NurbsSurface::removeChild(const SFAbstractNode &N)
{
  if (_texCoord == N)
  {
    setTexCoord(0);
    return true;
  } 
  if (N != 0)
    cx3d << "X3D::NurbsSurface::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::NurbsSurface::setChild : cannot add a NULL node!" << endl;
   
  
  return false;
}

void NurbsSurface::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("controlPoint");
  if (index != -1)
    _controlPoint = MFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("uTesselation");
  if (index != -1)
    _uTesselation = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("vTesselation");
  if (index != -1)
    _vTesselation = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("weight");
  if (index != -1)
    _weight = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("ccw");
  if (index != -1)
    _ccw = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("solid");
  if (index != -1)
    _solid = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("uDimension");
  if (index != -1)
    _uDimension = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("vDimension");
  if (index != -1)
    _vDimension = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("uKnot");
  if (index != -1)
    _uKnot = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("vKnot");
  if (index != -1)
    _vKnot = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("uOrder");
  if (index != -1)
    _uOrder = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("vOrder");
  if (index != -1)
    _vOrder = element->getAttribute(index).toInt();
}

SFString &NurbsSurface::write(SFString &output) const
{
  if (!_controlPoint.empty())
    output += " controlPoint=\"" + toSFString(_controlPoint) + "\"";
  if (_uTesselation != 0)
    output += " uTesselation=\"" + toSFString(_uTesselation) + "\"";
  if (_vTesselation != 0)
    output += " vTesselation=\"" + toSFString(_vTesselation) + "\"";
  if (!_weight.empty())
    output += " weight=\"" + toSFString(_weight) + "\"";
  if (!_ccw)
    output += " ccw=\"FALSE\"";
  if (!_solid)
    output += " solid=\"FALSE\"";
  if (_uDimension != 0)
    output += " uDimension=\"" + toSFString(_uDimension) + "\"";
  if (_vDimension != 0)
    output += " vDimension=\"" + toSFString(_vDimension) + "\"";
  if (!_uKnot.empty())
    output += " uKnot=\"" + toSFString(_uKnot) + "\"";
  if (!_vKnot.empty())
    output += " vKnot=\"" + toSFString(_vKnot) + "\"";
  if (_uOrder != 3)
    output += " uOrder=\"" + toSFString(_uOrder) + "\"";
  if (_vOrder != 3)
    output += " vOrder=\"" + toSFString(_vOrder) + "\"";
      
  return output;
}

}
}
